@@ -2,7 +2,7 @@
  *  Impulse Response Processor XMMS plugin
  *  Low Latency Version
  *
- *  Copyright (C) 2006-2014 Teru Kamogashira
+ *  Copyright (C) 2006-2018 Teru Kamogashira
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 #include <gtk/gtk.h>
 #include <libsamplerate2/samplerate2.h>
 #include <sndfile.h>
-#include <freeverb/irmodel.hpp>
+#include <freeverb/irmodel1.hpp>
 #include <freeverb/irmodel2.hpp>
 #include <freeverb/irmodel2zl.hpp>
 #include <freeverb/irmodel3.hpp>
@@ -80,7 +80,7 @@ extern "C" {
 
 #ifdef PLUGDOUBLE
 typedef fv3::irbase_ IRBASE;
-typedef fv3::irmodel_ IRMODEL;
+typedef fv3::irmodel1_ IRMODEL1;
 typedef fv3::irmodel2_ IRMODEL2;
 typedef fv3::irmodel2zl_ IRMODEL2ZL;
 typedef fv3::irmodel3_ IRMODEL3;
@@ -93,7 +93,7 @@ typedef fv3::CFileLoader_ CFILELOADER;
 typedef fv3::slot_ SLOTP;
 #else
 typedef fv3::irbase_f IRBASE;
-typedef fv3::irmodel_f IRMODEL;
+typedef fv3::irmodel1_f IRMODEL1;
 typedef fv3::irmodel2_f IRMODEL2;
 typedef fv3::irmodel2zl_f IRMODEL2ZL;
 typedef fv3::irmodel3_f IRMODEL3;
@@ -141,7 +141,7 @@ private:
   IRBASE * new_model(const char * type)
   {
     IRBASE * model = NULL;
-    if(strcmp(type, "irmodel") == 0)    model = new IRMODEL;
+    if(strcmp(type, "irmodel1") == 0)    model = new IRMODEL1;
     if(strcmp(type, "irmodel2") == 0)   model = new IRMODEL2;
     if(strcmp(type, "irmodel2zl") == 0) model = new IRMODEL2ZL;
     if(strcmp(type, "irmodel3") == 0)   model = new IRMODEL3;
@@ -164,36 +164,7 @@ static const char *about_text =
 #else
   "Single Precision Version\n"
 #endif
-  "SIMD: "
-#ifdef ENABLE_3DNOW
-  "3DNow! "
-#endif
-#ifdef ENABLE_SSE
-  "SSE_V1 "
-#endif
-#ifdef ENABLE_SSE_V2
-  "SSE_V2 "
-#endif
-#ifdef ENABLE_SSE2
-  "SSE2 "
-#endif
-#ifdef ENABLE_SSE3
-  "SSE3 "
-#endif
-#ifdef ENABLE_SSE4
-  "SSE4 "
-#endif
-#ifdef ENABLE_AVX
-  "AVX "
-#endif
-#ifdef ENABLE_FMA3
-  "FMA3 "
-#endif
-#ifdef ENABLE_FMA4
-  "FMA4 "
-#endif
-  "\n"
-  "Copyright (C) 2006-2014 Teru Kamogashira\n"
+  "Copyright (C) 2006-2018 Teru Kamogashira\n"
   "http://www.nongnu.org/freeverb3/";
 
 static const char *productString = "Freeverb3 "VERSION" [Impulser V2]";
@@ -225,9 +196,9 @@ static gboolean gdither_on = FALSE;
 
 static const int presetIRModelMax = 5;
 static const char * presetIRModelString[] =
-  { "fv3::irmodel2", "fv3::irmodel3 (Zero Latency)", "fv3::irmodel2zl (Zero Latency)", "fv3::irmodel", "fv3::irmodel3p (Zero Latency|Pthread)",};
+  { "fv3::irmodel2", "fv3::irmodel3 (Zero Latency)", "fv3::irmodel2zl (Zero Latency)", "fv3::irmodel1", "fv3::irmodel3p (Zero Latency|Pthread)",};
 static const char * presetIRModelValue[] =
-  {"irmodel2", "irmodel3", "irmodel2zl", "irmodel", "irmodel3p"};
+  {"irmodel2", "irmodel3", "irmodel2zl", "irmodel1", "irmodel3p"};
 static int conf_rev_zl = 1;
 
 // SLOT1 DRY + WET
@@ -343,7 +314,7 @@ static int store_inf(const char * file)
     }
   float second = (float)rsfinfo.frames/(float)rsfinfo.samplerate;
   sprintf(inf, "%lld[samples] %d[Hz] %d[Ch] %f[s]",
-	  (long long int)rsfinfo.frames, rsfinfo.samplerate, rsfinfo.channels, second);
+          (long long int)rsfinfo.frames, rsfinfo.samplerate, rsfinfo.channels, second);
   sf_close(sndFile);
   return 0;
 }
@@ -404,9 +375,9 @@ static void select_file(GtkWindow *parent)
 {
   std::string fc_filename;
   GtkWidget *file_dialog = gtk_file_chooser_dialog_new("Please select a Impulse Response wav file.", parent,
-						       GTK_FILE_CHOOSER_ACTION_OPEN,
-						       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						       GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, (char*)NULL);
+                                                       GTK_FILE_CHOOSER_ACTION_OPEN,
+                                                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                                       GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, (char*)NULL);
 
   gtk_widget_set_size_request(file_dialog, 900, 700);
   gtk_window_set_position(GTK_WINDOW(file_dialog), GTK_WIN_POS_CENTER);
@@ -643,9 +614,9 @@ static void * make_config_widget ()
 #define LABELS 14 // size of labels
 #define SHOWLABEL 6 // size of label which do not have gtk_adjustment
   const char * labels[64] = {"wet [dB]", "*dry [dB]", "1-pole LPF", "1-zero HPF",
-			     "width", "stretch [sqrt(2)^]", "ir limit [%]", "initial delay [ms]",
-			     "impulse file:", "pcm inf.", "mono/stereo slot",
-			     "*fragment size (<irmodel2|>irmodel3)", "*ir model type", "*dithering mode",};
+                             "width", "stretch [sqrt(2)^]", "ir limit [%]", "initial delay [ms]",
+                             "impulse file:", "pcm inf.", "mono/stereo slot",
+                             "*fragment size (<irmodel2|>irmodel3)", "*ir model type", "*dithering mode",};
   
   table = gtk_table_new(LABELS, 5, FALSE);
   gtk_table_set_col_spacings(GTK_TABLE(table), 1);
@@ -868,7 +839,7 @@ static gboolean init(void)
 #endif
   fprintf(stderr, "Impulser2: init()\n");
   fprintf(stderr, "SIMDFlag: 0x%08x\n", UTILS::getSIMDFlag());
-  if(UTILS::getSIMDFlag()&FV3_FLAG_SSE)
+  if(UTILS::getSIMDFlag()&FV3_X86SIMD_FLAG_SSE)
     {
       fprintf(stderr, "MXCSRMASK: 0x%08x\n", UTILS::getMXCSR_MASK());
       fprintf(stderr, "MXCSR: 0x%08x\n", UTILS::getMXCSR());
@@ -915,7 +886,7 @@ static gboolean init(void)
 static void cleanup(void)
 {
   fprintf(stderr, "Impulser2: cleanup()\n");
-  if(UTILS::getSIMDFlag()&FV3_FLAG_SSE) fprintf(stderr, "MXCSR: 0x%08x\n", UTILS::getMXCSR());
+  if(UTILS::getSIMDFlag()&FV3_X86SIMD_FLAG_SSE) fprintf(stderr, "MXCSR: 0x%08x\n", UTILS::getMXCSR());
 
   pthread_mutex_lock(&plugin_mutex);
   plugin_available = false;
@@ -959,27 +930,27 @@ static void mod_samples_f(pfloat_t * iL, pfloat_t * iR, pfloat_t * oL, pfloat_t
     {
       fprintf(stderr, "Impulser2: mod_samples: Slot %d -> %d\n", (int)reverbVector->size(), slotNumber);
       if((int)reverbVector->size() < slotNumber) // increase slot
-	{
-	  SlotConfiguration slotC;
-	  slot_init(&slotC);
-	  while(1)
-	    {
-	      if((int)reverbVector->size() == slotNumber) break;
-	      IRBASE * model = reverbVector->push_back(presetIRModelValue[conf_rev_zl]);
-	      if(reverbVector->size() <= slotVector->size())
-		set_rt_reverb(model, &(*slotVector)[reverbVector->size()-1]);
-	      currentSlotVector->push_back(slotC);
-	    }
-	}
+        {
+          SlotConfiguration slotC;
+          slot_init(&slotC);
+          while(1)
+            {
+              if((int)reverbVector->size() == slotNumber) break;
+              IRBASE * model = reverbVector->push_back(presetIRModelValue[conf_rev_zl]);
+              if(reverbVector->size() <= slotVector->size())
+                set_rt_reverb(model, &(*slotVector)[reverbVector->size()-1]);
+              currentSlotVector->push_back(slotC);
+            }
+        }
       else // decrease slot
-	{
-	  while(1)
-	    {
-	      if((int)reverbVector->size() == slotNumber) break;
-	      reverbVector->pop_back();
-	      currentSlotVector->pop_back();
-	    }
-	}
+        {
+          while(1)
+            {
+              if((int)reverbVector->size() == slotNumber) break;
+              reverbVector->pop_back();
+              currentSlotVector->pop_back();
+            }
+        }
     }
   
   // reset all if srate or latency have been changed
@@ -992,11 +963,11 @@ static void mod_samples_f(pfloat_t * iL, pfloat_t * iR, pfloat_t * oL, pfloat_t
       slot_init(&slotC);
       currentSlotVector->clear();
       for(int i = 0;i < slotNumber;i ++)
-	{
-	  currentSlotVector->push_back(slotC);
-	  reverbVector->assign(i, presetIRModelValue[conf_rev_zl]);
-	  set_rt_reverb(reverbVector->at(i), &(*slotVector)[i]);
-	}
+        {
+          currentSlotVector->push_back(slotC);
+          reverbVector->assign(i, presetIRModelValue[conf_rev_zl]);
+          set_rt_reverb(reverbVector->at(i), &(*slotVector)[i]);
+        }
       fprintf(stderr, "Impulser2: mod_samples: vector %d, cvector %d\n", (int)slotVector->size(), (int)currentSlotVector->size());
     }
   
@@ -1005,98 +976,95 @@ static void mod_samples_f(pfloat_t * iL, pfloat_t * iR, pfloat_t * oL, pfloat_t
       if(reverbVector->size() > slotVector->size()) return;
       // changed stretch/limit reload
       if((*currentSlotVector)[i].stretch != (*slotVector)[i].stretch||
-	 (*currentSlotVector)[i].limit != (*slotVector)[i].limit||
-	 strcmp((*currentSlotVector)[i].filename.c_str(), (*slotVector)[i].filename.c_str()) != 0)
-	{
-	  fprintf(stderr, "Impulser2: mod_samples: typeid=%s\n", typeid(*(*reverbVector)[i]).name());
-	  (*currentSlotVector)[i].stretch = (*slotVector)[i].stretch;
-	  (*currentSlotVector)[i].limit = (*slotVector)[i].limit;
-	  if(latencyIndex >= presetLatencyMax) conf_latency_index = latencyIndex = 0;
-	  if(typeid(*(*reverbVector)[i]) == typeid(IRMODEL2))
-	    {
-	      fprintf(stderr, "Impulser2: mod_samples: irmodel2 %ld\n", presetLatencyValue[latencyIndex]);
-	      dynamic_cast<IRMODEL2*>((*reverbVector)[i])->setFragmentSize(presetLatencyValue[latencyIndex]);
-	    }
-	  if(typeid(*(*reverbVector)[i]) == typeid(IRMODEL2ZL))
-	    {
-	      fprintf(stderr, "Impulser2: mod_samples: irmodel2zl %ld\n", presetLatencyValue[latencyIndex]);
-	      dynamic_cast<IRMODEL2ZL*>((*reverbVector)[i])->setFragmentSize(presetLatencyValue[latencyIndex]);
-	    }
-	  if(typeid(*(*reverbVector)[i]) == typeid(IRMODEL3)
+         (*currentSlotVector)[i].limit != (*slotVector)[i].limit||
+         strcmp((*currentSlotVector)[i].filename.c_str(), (*slotVector)[i].filename.c_str()) != 0)
+        {
+          fprintf(stderr, "Impulser2: mod_samples: typeid=%s\n", typeid(*(*reverbVector)[i]).name());
+          (*currentSlotVector)[i].stretch = (*slotVector)[i].stretch;
+          (*currentSlotVector)[i].limit = (*slotVector)[i].limit;
+          if(latencyIndex >= presetLatencyMax) conf_latency_index = latencyIndex = 0;
+          if(typeid(*(*reverbVector)[i]) == typeid(IRMODEL2))
+            {
+              fprintf(stderr, "Impulser2: mod_samples: irmodel2 %ld\n", presetLatencyValue[latencyIndex]);
+              dynamic_cast<IRMODEL2*>((*reverbVector)[i])->setFragmentSize(presetLatencyValue[latencyIndex]);
+            }
+          if(typeid(*(*reverbVector)[i]) == typeid(IRMODEL2ZL))
+            {
+              fprintf(stderr, "Impulser2: mod_samples: irmodel2zl %ld\n", presetLatencyValue[latencyIndex]);
+              dynamic_cast<IRMODEL2ZL*>((*reverbVector)[i])->setFragmentSize(presetLatencyValue[latencyIndex]);
+            }
+          if(typeid(*(*reverbVector)[i]) == typeid(IRMODEL3)
 #ifdef ENABLE_PTHREAD
-	     ||typeid(*(*reverbVector)[i]) == typeid(IRMODEL3P)
+             ||typeid(*(*reverbVector)[i]) == typeid(IRMODEL3P)
 #endif
-	     )
-	    {
-	      fprintf(stderr, "Impulser2: mod_samples: irmodel3/p %ld %ld\n", presetLatencyValue1[latencyIndex], presetLatencyValue2[latencyIndex]);
-	      dynamic_cast<IRMODEL3*>((*reverbVector)[i])->setFragmentSize(presetLatencyValue1[latencyIndex], presetLatencyValue2[latencyIndex]);
-	    }
-	  CFILELOADER fileLoader;
-	  double l_stretch = std::pow(static_cast<double>(std::sqrt(2)), static_cast<double>((*slotVector)[i].stretch));
-	  int ret = fileLoader.load((*slotVector)[i].filename.c_str(), srate, l_stretch, (*slotVector)[i].limit, SRC_SINC_BEST_QUALITY);
+             )
+            {
+              fprintf(stderr, "Impulser2: mod_samples: irmodel3/p %ld %ld\n", presetLatencyValue1[latencyIndex], presetLatencyValue2[latencyIndex]);
+              dynamic_cast<IRMODEL3*>((*reverbVector)[i])->setFragmentSize(presetLatencyValue1[latencyIndex], presetLatencyValue2[latencyIndex]);
+            }
+          CFILELOADER fileLoader;
+          double l_stretch = std::pow(static_cast<double>(std::sqrt(2)), static_cast<double>((*slotVector)[i].stretch));
+          int ret = fileLoader.load((*slotVector)[i].filename.c_str(), srate, l_stretch, (*slotVector)[i].limit, SRC_SINC_BEST_QUALITY);
 	  
-	  if(ret == 0)
-	    {
-	      (*reverbVector)[i]->loadImpulse(fileLoader.out.L, fileLoader.out.R, fileLoader.out.getsize());
-	      (*currentSlotVector)[i].filename = (*slotVector)[i].filename;
-	      (*currentSlotVector)[i].valid = 1;
-	      fprintf(stderr, "Impulser2: mod_samples: Slot[%d] \"%s\"(%ld)\n", i, (*slotVector)[i].filename.c_str(), (*reverbVector)[i]->getSampleSize());
-	    }
-	  else
-	    {
-	      fprintf(stderr, "Impulser2: mod_samples: Slot[%d] IR load fail! ret=%d ", i, ret);
-	      fprintf(stderr, "<%s>\n", fileLoader.errstr());
-	      (*currentSlotVector)[i].valid = 0;
-	    }
-	}
+          if(ret == 0)
+            {
+              (*reverbVector)[i]->loadImpulse(fileLoader.out.L, fileLoader.out.R, fileLoader.out.getsize());
+              (*currentSlotVector)[i].filename = (*slotVector)[i].filename;
+              (*currentSlotVector)[i].valid = 1;
+              fprintf(stderr, "Impulser2: mod_samples: Slot[%d] \"%s\"(%ld)\n", i, (*slotVector)[i].filename.c_str(), (*reverbVector)[i]->getImpulseSize());
+            }
+          else
+            {
+              fprintf(stderr, "Impulser2: mod_samples: Slot[%d] IR load fail! ret=%d ", i, ret);
+              fprintf(stderr, "<%s>\n", fileLoader.errstr());
+              (*currentSlotVector)[i].valid = 0;
+            }
+        }
 
       // delay
       long iDelay = (int)((float)StreamFs*(*slotVector)[i].idelay/1000.0f);
       if((*reverbVector)[i]->getInitialDelay() != iDelay)
-	{
-	  fprintf(stderr, "Impulser2: mod_samples: InitialDelay[%d] %ld -> %ld\n", i, (*reverbVector)[i]->getInitialDelay(), iDelay);
-	  (*reverbVector)[i]->setInitialDelay(iDelay);
-	}
+        {
+          fprintf(stderr, "Impulser2: mod_samples: InitialDelay[%d] %ld -> %ld\n", i, (*reverbVector)[i]->getInitialDelay(), iDelay);
+          (*reverbVector)[i]->setInitialDelay(iDelay);
+        }
       
       // slot
       if((*slotVector)[i].i1o2_index < presetSlotModeMax&&(*slotVector)[i].i1o2_index >= 0)
-	{
-	  if((*currentSlotVector)[i].i1o2_index != presetSlotModeValue[(*slotVector)[i].i1o2_index])
-	    {
-	      (*currentSlotVector)[i].i1o2_index = presetSlotModeValue[(*slotVector)[i].i1o2_index];
-	      fprintf(stderr, "Impulser2: mod_samples: conf_i1o2[%d] -> %d\n", i, (*currentSlotVector)[i].i1o2_index);
-	    }
-	}
+        {
+          if((*currentSlotVector)[i].i1o2_index != presetSlotModeValue[(*slotVector)[i].i1o2_index])
+            {
+              (*currentSlotVector)[i].i1o2_index = presetSlotModeValue[(*slotVector)[i].i1o2_index];
+              fprintf(stderr, "Impulser2: mod_samples: conf_i1o2[%d] -> %d\n", i, (*currentSlotVector)[i].i1o2_index);
+            }
+        }
     }
   
   validNumber = 0;
   for(int i = 0;i < (int)reverbVector->size();i ++)
     {
       if((*currentSlotVector)[i].valid == 1&&(int)slotVector->size() > i)
-	{
-	  unsigned options = FV3_IR_DEFAULT;
-	  if((*currentSlotVector)[i].i1o2_index == 1)
-	    options |= FV3_IR_MONO2STEREO;
-	  if((*currentSlotVector)[i].i1o2_index == 3)
-	    options |= FV3_IR_SWAP_LR;
-	  if((*slotVector)[i].wet <= MIN_DB)
-	    options |= FV3_IR_MUTE_WET;
-	  if((*slotVector)[0].dry <= MIN_DB||i > 0)
-	    options |= FV3_IR_MUTE_DRY;
-	  if((*slotVector)[i].lpf <= 0.0&&(*slotVector)[i].hpf <= 0.0)
-	    options |= FV3_IR_SKIP_FILTER;
-	  if((int)reverbVector->size() > i)
-	    {
-	      if((*reverbVector)[i]->getSampleSize() > 0)
-		{
-		  if(i == 0)
-		    (*reverbVector)[i]->processreplace(iL,iR,oL,oR,length,options);
-		  else
-		    (*reverbVector)[i]->processreplace(iL,iR,oL,oR,length,options|FV3_IR_SKIP_INIT);
-		  validNumber ++;
-		}
-	    }
-	}
+        {
+          unsigned options = FV3_IR_DEFAULT;
+          if((*currentSlotVector)[i].i1o2_index == 1) options |= FV3_IR_MONO2STEREO;
+          if((*currentSlotVector)[i].i1o2_index == 3) options |= FV3_IR_SWAP_LR;
+          if((*slotVector)[i].wet <= MIN_DB)          options |= FV3_IR_MUTE_WET;
+          if((*slotVector)[0].dry <= MIN_DB||i > 0)   options |= FV3_IR_MUTE_DRY;
+          if((*slotVector)[i].lpf <= 0.0&&(*slotVector)[i].hpf <= 0.0) options |= FV3_IR_SKIP_FILTER;
+          if((int)reverbVector->size() > i)
+            {
+              if((*reverbVector)[i]->getImpulseSize() > 0)
+                {
+                  if(typeid(*(*reverbVector)[i]) == typeid(IRMODEL1))
+                    options = FV3_IR_DEFAULT;
+                  if(i == 0)
+                    (*reverbVector)[i]->processreplace(iL,iR,oL,oR,length,options);
+                  else
+                    (*reverbVector)[i]->processreplace(iL,iR,oL,oR,length,options|FV3_IR_SKIP_INIT);
+                  validNumber ++;
+                }
+            }
+        }
     }  
   pthread_mutex_unlock(&plugin_mutex);
 }
