@@ -1,7 +1,7 @@
 /**
  *  FFT impulse fragment square multiplier
  *
- *  Copyright (C) 2006-2014 Teru Kamogashira
+ *  Copyright (C) 2006-2018 Teru Kamogashira
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@ FV3_(fragfft)::FV3_(fragfft)()
 {
   fragmentSize = 0;
   simdSize = 1;
-  setSIMD(0);
+  setSIMD(0,0);
 }
 
 FV3_(fragfft)::FV3_(~fragfft)()
@@ -36,64 +36,9 @@ FV3_(fragfft)::FV3_(~fragfft)()
   freeFFT();
 }
 
-void FV3_(fragfft)::setSIMD(uint32_t simdFlag)
+long FV3_(fragfft)::getSIMDSize()
 {
-  if(simdFlag == 0) simdFlag = FV3_(utils)::getSIMDFlag();
-  simdSize = 1;
-
-#ifdef LIBFV3_FLOAT
-#if defined(ENABLE_3DNOW)
-  if((simdFlag&FV3_FLAG_3DNOW))
-    simdSize = 2;
-#endif
-#if defined(ENABLE_SSE)
-  if((simdFlag&FV3_FLAG_SSE))
-    simdSize = 4;
-#endif
-#if defined(ENABLE_SSE_V2)||defined(ENABLE_SSE2)
-  if((simdFlag&FV3_FLAG_SSE))
-    simdSize = 1;
-#endif
-#if defined(ENABLE_SSE3)||defined(ENABLE_SSE4)
-  if((simdFlag&FV3_FLAG_SSE3))
-    simdSize = 1;
-#endif
-#if defined(ENABLE_AVX)
-  if((simdFlag&FV3_FLAG_AVX))
-    simdSize = 8;
-#endif
-#if defined(ENABLE_FMA3)
-  if((simdFlag&FV3_FLAG_FMA3))
-    simdSize = 8;
-#endif
-#if defined(ENABLE_FMA4)
-  if((simdFlag&FV3_FLAG_FMA4))
-    simdSize = 8;
-#endif
-#endif
-
-#ifdef LIBFV3_DOUBLE
-#if defined(ENABLE_SSE2)||defined(ENABLE_SSE3)
-  if((simdFlag&FV3_FLAG_SSE2))
-    simdSize = 2;
-#endif
-#if defined(ENABLE_SSE4)
-  if((simdFlag&FV3_FLAG_SSE4_1))
-    simdSize = 1;
-#endif
-#if defined(ENABLE_AVX)
-  if((simdFlag&FV3_FLAG_AVX))
-    simdSize = 4;
-#endif
-#if defined(ENABLE_FMA3)
-  if((simdFlag&FV3_FLAG_FMA3))
-    simdSize = 4;
-#endif
-#if defined(ENABLE_FMA4)
-  if((simdFlag&FV3_FLAG_FMA4))
-    simdSize = 4;
-#endif
-#endif
+  return simdSize;
 }
 
 long FV3_(fragfft)::getFragmentSize()
@@ -109,8 +54,7 @@ void FV3_(fragfft)::allocFFT(long size, unsigned fftflags)
 #endif
   if(FV3_IR_Min_FragmentSize > size)
     {
-      std::fprintf(stderr, "fragfft::allocFFT(size=%ld): fragmentSize(>%d) is too small! \n",
-		   size, FV3_IR_Min_FragmentSize);
+      std::fprintf(stderr, "fragfft::allocFFT(size=%ld): fragmentSize (>%d) is too small! \n", size, FV3_IR_Min_FragmentSize);
       throw std::bad_alloc();
     }
   if(size != FV3_(utils)::checkPow2(size))
@@ -119,21 +63,17 @@ void FV3_(fragfft)::allocFFT(long size, unsigned fftflags)
       throw std::bad_alloc();
     }
   freeFFT();
-  fftOrig.alloc(2*size, 2);
+  fftOrig.alloc(2*size, 1);
+  planRevrL = FFTW_(plan_r2r_1d)(2*size, fftOrig.L, fftOrig.L, FFTW_HC2R, fftflags);
   planOrigL = FFTW_(plan_r2r_1d)(2*size, fftOrig.L, fftOrig.L, FFTW_R2HC, fftflags);
-  planOrigR = FFTW_(plan_r2r_1d)(2*size, fftOrig.R, fftOrig.R, FFTW_R2HC, fftflags);
-  planRevL = FFTW_(plan_r2r_1d)(2*size, fftOrig.L, fftOrig.L, FFTW_HC2R, fftflags);
-  planRevR = FFTW_(plan_r2r_1d)(2*size, fftOrig.R, fftOrig.R, FFTW_HC2R, fftflags);
   fragmentSize = size;
 }
 
 void FV3_(fragfft)::freeFFT()
 {
   if(fragmentSize == 0) return;
-  FFTW_(destroy_plan)(planRevL);
-  FFTW_(destroy_plan)(planRevR);
+  FFTW_(destroy_plan)(planRevrL);
   FFTW_(destroy_plan)(planOrigL);
-  FFTW_(destroy_plan)(planOrigR);
   fftOrig.free();
   fragmentSize = 0;
 }
@@ -167,29 +107,13 @@ void FV3_(fragfft)::R2SA(const fv3_float_t * in, fv3_float_t * out, long n)
     }
 }
 
-void FV3_(fragfft)::R2HC(const fv3_float_t * iL, const fv3_float_t * iR,
-			 fv3_float_t * oL, fv3_float_t * oR)
+void FV3_(fragfft)::R2HC(const fv3_float_t * iL, fv3_float_t * oL)
 {
   if(fragmentSize == 0) return;
-#pragma omp parallel
-#pragma omp sections
-  {
-#pragma omp section
-    {
-      FV3_(utils)::mute(fftOrig.L+fragmentSize, fragmentSize);
-      std::memcpy(fftOrig.L, iL, sizeof(fv3_float_t)*fragmentSize);
-      FFTW_(execute)(planOrigL);
-      R2SA(fftOrig.L, oL, fragmentSize*2);
-    }
-#pragma omp section
-    {
-      FV3_(utils)::mute(fftOrig.R+fragmentSize, fragmentSize);
-      std::memcpy(fftOrig.R, iR, sizeof(fv3_float_t)*fragmentSize);
-      FFTW_(execute)(planOrigR);
-      R2SA(fftOrig.R, oR, fragmentSize*2);
-    }
-  }
-#pragma omp barrier
+  FV3_(utils)::mute(fftOrig.L+fragmentSize, fragmentSize);
+  std::memcpy(fftOrig.L, iL, sizeof(fv3_float_t)*fragmentSize);
+  FFTW_(execute)(planOrigL);
+  R2SA(fftOrig.L, oL, fragmentSize*2);
   return;
 }
 
@@ -224,26 +148,12 @@ void FV3_(fragfft)::SA2R(const fv3_float_t * in, fv3_float_t * out, long n)
     }
 }
 
-void FV3_(fragfft)::HC2R(const fv3_float_t * iL, const fv3_float_t * iR,
-			 fv3_float_t * oL, fv3_float_t * oR)
+void FV3_(fragfft)::HC2R(const fv3_float_t * iL, fv3_float_t * oL)
 {
   if(fragmentSize == 0) return;
-#pragma omp parallel
-#pragma omp sections
-  {
-#pragma omp section
-    {
-      SA2R(iL, fftOrig.L, fragmentSize*2);
-      FFTW_(execute)(planRevL);
-      for(long i = 0;i < fragmentSize*2;i ++) oL[i] += fftOrig.L[i];
-    }
-#pragma omp section
-    {
-      SA2R(iR, fftOrig.R, fragmentSize*2);
-      FFTW_(execute)(planRevR);
-      for(long i = 0;i < fragmentSize*2;i ++) oR[i] += fftOrig.R[i];
-    }
-  }
+  SA2R(iL, fftOrig.L, fragmentSize*2);
+  FFTW_(execute)(planRevrL);
+  for(long i = 0;i < fragmentSize*2;i ++) oL[i] += fftOrig.L[i];
   return;
 }
 
@@ -253,7 +163,7 @@ FV3_(frag)::FV3_(frag)()
 {
   fragmentSize = 0;
   fftImpulse.L = fftImpulse.R = NULL;
-  setSIMD(0);
+  setSIMD(0,0);
 }
 
 FV3_(frag)::FV3_(~frag)()
@@ -261,14 +171,13 @@ FV3_(frag)::FV3_(~frag)()
   unloadImpulse();
 }
 
-void FV3_(frag)::loadImpulse(const fv3_float_t * L, const fv3_float_t * R, long size, long limit, unsigned fftflags)
+void FV3_(frag)::loadImpulse(const fv3_float_t * L, long size, long limit, unsigned fftflags)
 		throw(std::bad_alloc)
 {
-  this->loadImpulse(L,R,size,limit,fftflags,NULL,NULL);
+  this->loadImpulse(L,size,limit,fftflags,NULL);
 }
 
-void FV3_(frag)::loadImpulse(const fv3_float_t * L, const fv3_float_t * R, long size, long limit, unsigned fftflags,
-			     fv3_float_t * preAllocatedL, fv3_float_t * preAllocatedR)
+void FV3_(frag)::loadImpulse(const fv3_float_t * L, long size, long limit, unsigned fftflags, fv3_float_t * preAllocatedL)
 		throw(std::bad_alloc)
 {
 #ifdef DEBUG
@@ -276,8 +185,7 @@ void FV3_(frag)::loadImpulse(const fv3_float_t * L, const fv3_float_t * R, long
 #endif
   if(FV3_IR_Min_FragmentSize > size)
     {
-      std::fprintf(stderr, "frag::loadImpulse(f=%ld,l=%ld): fragmentSize(>%d) is too small.\n",
-		   size, limit, FV3_IR_Min_FragmentSize);
+      std::fprintf(stderr, "frag::loadImpulse(f=%ld,l=%ld): fragmentSize(>%d) is too small.\n", size, limit, FV3_IR_Min_FragmentSize);
       throw std::bad_alloc();
     }
   if(size != FV3_(utils)::checkPow2(size))
@@ -289,22 +197,18 @@ void FV3_(frag)::loadImpulse(const fv3_float_t * L, const fv3_float_t * R, long
   if(size < limit) limit = size;
   unloadImpulse();
   FV3_(fragfft) fragFFT;
+  fragFFT.setSIMD(simdFlag1, simdFlag2);
   // impulse = [_Re_ impulse...< limit 0...0 (size)][_Im_ 0...0 (size*2)]
   FV3_(slot) impulse;
-  impulse.alloc(size, 2);
-  
-  for(long i = 0;i < limit;i ++)
-    {
-      impulse.L[i] = L[i] / (fv3_float_t)(size*2);
-      impulse.R[i] = R[i] / (fv3_float_t)(size*2);
-    }
+  impulse.alloc(size, 1);
+  for(long i = 0;i < limit;i ++){ impulse.L[i] = L[i] / (fv3_float_t)(size*2); }
 
   try
     {
-      if(preAllocatedL == NULL||preAllocatedR == NULL)
+      if(preAllocatedL == NULL)
 	allocImpulse(size);
       else
-	registerPreallocatedBlock(preAllocatedL, preAllocatedR, size);
+	registerPreallocatedBlock(preAllocatedL, size);
       fragFFT.allocFFT(size, fftflags);
     }
   catch(std::bad_alloc)
@@ -312,15 +216,14 @@ void FV3_(frag)::loadImpulse(const fv3_float_t * L, const fv3_float_t * R, long
       unloadImpulse();
       throw;
     }
-  fragFFT.R2HC(impulse.L, impulse.R, fftImpulse.L, fftImpulse.R);
+  fragFFT.R2HC(impulse.L, fftImpulse.L);
 }
 
-void FV3_(frag)::registerPreallocatedBlock(fv3_float_t * _L, fv3_float_t * _R, long size)
+void FV3_(frag)::registerPreallocatedBlock(fv3_float_t * _L, long size)
 {
   freeImpulse();
   fragmentSize = size;
   fftImpulse.L = _L;
-  fftImpulse.R = _R;
 }
 
 void FV3_(frag)::allocImpulse(long size)
@@ -328,7 +231,7 @@ void FV3_(frag)::allocImpulse(long size)
 {
   freeImpulse();
   fragmentSize = size;
-  fftImpulse.alloc(2*size, 2);
+  fftImpulse.alloc(2*size, 1);
 }
 
 void FV3_(frag)::freeImpulse()
@@ -344,7 +247,7 @@ void FV3_(frag)::unloadImpulse()
 }
 
 #ifdef LIBFV3_FLOAT
-#if defined(ENABLE_FMA4)
+#if defined(ENABLE_X86SIMD)
 static void MULT_M_F_FMA4(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -397,9 +300,7 @@ static void MULT_M_F_FMA4(const fv3_float_t * iL, const fv3_float_t * fL, fv3_fl
      : "q"(oL), "q"(iL), "q"(fL), "r"(n/16), "r"(soL)
      : "memory");
 }
-#endif
 
-#if defined(ENABLE_FMA3)
 static void MULT_M_F_FMA3(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -452,9 +353,7 @@ static void MULT_M_F_FMA3(const fv3_float_t * iL, const fv3_float_t * fL, fv3_fl
      : "q"(oL), "q"(iL), "q"(fL), "r"(n/16), "r"(soL)
      : "memory");
 }
-#endif
 
-#if defined(ENABLE_AVX)
 static void MULT_M_F_AVX(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -513,9 +412,7 @@ static void MULT_M_F_AVX(const fv3_float_t * iL, const fv3_float_t * fL, fv3_flo
      : "q"(oL), "q"(iL), "q"(fL), "r"(n/16), "r"(soL)
      : "memory");
 }
-#endif
 
-#if defined(ENABLE_SSE3)||defined(ENABLE_SSE4)
 static void MULT_M_F_SSE3(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -565,9 +462,7 @@ static void MULT_M_F_SSE3(const fv3_float_t * iL, const fv3_float_t * fL, fv3_fl
      : "q"(oL), "q"(iL), "q"(fL), "r"(n/4), "r"(soL)
      : "memory");
 }
-#endif
- 
-#if defined(ENABLE_SSE_V2)||defined(ENABLE_SSE2)
+
 static void MULT_M_F_SSE_V2(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -628,9 +523,7 @@ static void MULT_M_F_SSE_V2(const fv3_float_t * iL, const fv3_float_t * fL, fv3_
      : "q"(oL), "q"(iL), "q"(fL), "r"(n/4), "r"(soL)
      : "memory");
 }
-#endif
 
-#if defined(ENABLE_SSE)
 static void MULT_M_F_SSE(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -679,9 +572,7 @@ static void MULT_M_F_SSE(const fv3_float_t * iL, const fv3_float_t * fL, fv3_flo
      : "q"(oL), "q"(iL), "q"(fL), "r"(n/4), "r"(soL)
      : "memory");
 }
-#endif
 
-#if defined(ENABLE_3DNOW)
 static void MULT_M_F_3DNOW(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -736,7 +627,7 @@ static void MULT_M_F_3DNOW(const fv3_float_t * iL, const fv3_float_t * fL, fv3_f
 #endif
 
 #ifdef LIBFV3_DOUBLE
-#if defined(ENABLE_FMA4)
+#if defined(ENABLE_X86SIMD)
 static void MULT_M_D_FMA4(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -789,9 +680,7 @@ static void MULT_M_D_FMA4(const fv3_float_t * iL, const fv3_float_t * fL, fv3_fl
      : "q"(oL), "q"(iL), "q"(fL), "r"(n/8), "r"(soL)
      : "memory");
 }
-#endif
 
-#if defined(ENABLE_FMA3)
 static void MULT_M_D_FMA3(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -844,9 +733,7 @@ static void MULT_M_D_FMA3(const fv3_float_t * iL, const fv3_float_t * fL, fv3_fl
      : "q"(oL), "q"(iL), "q"(fL), "r"(n/8), "r"(soL)
      : "memory");
 }
-#endif
 
-#if defined(ENABLE_AVX)
 static void MULT_M_D_AVX(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -905,9 +792,7 @@ static void MULT_M_D_AVX(const fv3_float_t * iL, const fv3_float_t * fL, fv3_flo
      : "q"(oL), "q"(iL), "q"(fL), "r"(n/8), "r"(soL)
      : "memory");
 }
-#endif
 
-#if defined(ENABLE_SSE4)
 static void MULT_M_D_SSE4(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -917,44 +802,43 @@ static void MULT_M_D_SSE4(const fv3_float_t * iL, const fv3_float_t * fL, fv3_fl
 {
   void * soL = NULL;
   __asm__ __volatile__
-    ("MULT_M_D_SSE4:          \n\t"
-     "movapd (%1), %%xmm7     \n\t" // double tL0 = oL[0] + iL[0] * fL[0]; double tL1 = oL[1] + iL[1] * fL[1];
+    ("movapd (%1), %%xmm7     \n\t" // double tL0 = oL[0] + iL[0] * fL[0]; double tL1 = oL[1] + iL[1] * fL[1];
      "mulpd  (%2), %%xmm7     \n\t"
      "addpd  (%0), %%xmm7     \n\t"
-     "mov    $0x80, %4        \n\t"
-     "pxor   %%xmm6, %%xmm6   \n\t"
-     "movd   %4, %%xmm6       \n\t"
      // The bitshift is not supported in SSE :-(
-     // ex.
+     // Usage example of the bitshift in SSE2:
      // pcmpeqd xmm0, xmm0 XMM0 = 0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFh
      // pslldq  xmm0, 3    XMM0 = 0FFFFFFFFFFFFFFFFFFFFFFFFFF000000h pslldq:sse2
-     "pslldq $0xf, %%xmm6  \n\t" // bitmask = xmm6 ([0]=0,[1]=-1)
-     // XMM6 = 8000000000000000 0000000000000000h
+     "xor    %4, %4           \n\t"
+     "mov    $0x80, %4        \n\t"
+     "xorps  %%xmm6, %%xmm6   \n\t"
+     "movd   %4, %%xmm6       \n\t" // XMM6 = 0000000000000000 0000000000000080h
+     "pslldq $0xf, %%xmm6     \n\t" // XMM6 = 8000000000000000 0000000000000000h ([0]=0,[1]=-1)
      "mov    %0, %4           \n\t" // save %0(oL)
      "MULT_M_D_SSE4_LOOP:     \n\t" // bitmask:xmm6
      "prefetchnta 0x80(%1)    \n\t"
-     "movapd     (%1), %%xmm0 \n\t" // xmm0,1,2
-     "movapd 0x10(%1), %%xmm3 \n\t" // xmm3,4,5
+     "movapd     (%1), %%xmm0 \n\t" // [1]xmm0,1,2 i(a,b)->xmm0
+     "movapd 0x10(%1), %%xmm3 \n\t" // [2]xmm3,4,5
      "add    $0x20, %1        \n\t"
      "prefetchnta 0x80(%2)    \n\t"
-     "movapd     (%2), %%xmm1 \n\t"
-     "movapd 0x10(%2), %%xmm4 \n\t"
+     "movapd     (%2), %%xmm1 \n\t" // [1] f(c,d)->xmm1
+     "movapd 0x10(%2), %%xmm4 \n\t" // [2]
      "add    $0x20, %2        \n\t"
-     "movapd %%xmm1, %%xmm2   \n\t"
-     "movapd %%xmm4, %%xmm5   \n\t"
+     "movapd %%xmm1, %%xmm2   \n\t" // [1] xmm2<-xmm1
+     "movapd %%xmm4, %%xmm5   \n\t" // [2]
      "add    $0x20, %0        \n\t"
-     "xorpd  %%xmm6, %%xmm1   \n\t"
+     "xorpd  %%xmm6, %%xmm1   \n\t" // [1] xmm1(c,-1*d)
      "xorpd  %%xmm6, %%xmm4   \n\t"
      "dppd   $0x31, %%xmm0, %%xmm1 \n\t" // (a,b)#(c,-d)->(ac-bd,0)
      "dppd   $0x31, %%xmm3, %%xmm4 \n\t"
-     "shufpd $0x3,  %%xmm2, %%xmm2 \n\t" // (c,d)->(d,c)
-     "shufpd $0x3,  %%xmm5, %%xmm5 \n\t"
+     "shufpd $0x1,  %%xmm2, %%xmm2 \n\t" // (c,d)->(d,c)
+     "shufpd $0x1,  %%xmm5, %%xmm5 \n\t"
      "dppd   $0x32, %%xmm0, %%xmm2 \n\t" // (a,b)#(d,c)->(0,ad+bc)
      "dppd   $0x32, %%xmm3, %%xmm5 \n\t"
      "xorpd  %%xmm1, %%xmm2    \n\t" // (ac-bd,ad+bc)
      "xorpd  %%xmm4, %%xmm5    \n\t"
      "prefetcht2 0x80(%0)      \n\t"
-     "addpd  -0x20(%0), %%xmm2  \n\t"
+     "addpd  -0x20(%0), %%xmm2  \n\t" // o += xmm2
      "addpd  -0x10(%0), %%xmm5  \n\t"
      "dec    %3                 \n\t"
      "movapd %%xmm2, -0x20(%0)  \n\t"
@@ -966,9 +850,7 @@ static void MULT_M_D_SSE4(const fv3_float_t * iL, const fv3_float_t * fL, fv3_fl
      : "q"(oL), "q"(iL), "q"(fL), "r"(n/2), "r"(soL)
      : "memory");
 }
-#endif
 
-#if defined(ENABLE_SSE2)||defined(ENABLE_SSE3)
 static void MULT_M_D_SSE2(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float_t * oL, long n)
 #ifdef __GNUC__
   __attribute__((noinline))
@@ -978,8 +860,7 @@ static void MULT_M_D_SSE2(const fv3_float_t * iL, const fv3_float_t * fL, fv3_fl
 {
   void * soL = NULL;
   __asm__ __volatile__
-    ("MULT_M_D_SSE2:            \n\t"
-     "mov    %0, %4           \n\t"
+    ("mov    %0, %4           \n\t"
      "movsd  (%1), %%xmm6     \n\t" // double tL0 = oL[0] + iL[0] * fL[0];
      "mulsd  (%2), %%xmm6     \n\t"
      "addsd  (%0), %%xmm6     \n\t"
@@ -1043,86 +924,117 @@ static void MULT_M_FPU(const fv3_float_t * iL, const fv3_float_t * fL, fv3_float
   oL[1] = tL1;
 }
 
-void FV3_(frag)::setSIMD(uint32_t simdFlag)
+uint32_t FV3_(fragfft)::getSIMD(uint32_t select)
 {
-  if(simdFlag == 0) simdFlag = FV3_(utils)::getSIMDFlag();
-  MULT_M = MULT_M_FPU;
+  if(select == 0) return simdFlag1;
+  if(select == 1) return simdFlag2;
+  return 0;
+}
+
+uint32_t FV3_(frag)::getSIMD(uint32_t select)
+{
+  if(select == 0) return simdFlag1;
+  if(select == 1) return simdFlag2;
+  return 0;
+}
+
+void FV3_(fragfft)::setSIMD(uint32_t flag1, uint32_t flag2)
+{
+  // flag1 == NULL or unsupported Instruction -> Autodetect
+  if(flag1 == FV3_X86SIMD_NULL)
+    simdFlag1 = FV3_(utils)::getSIMDFlag();
+  else if(!(flag1 & FV3_(utils)::getSIMDFlag()))
+    {
+      std::fprintf(stderr, "fragfft::setSIMD(%08x): not supported, autodetected.\n", flag1);
+      simdFlag1 = FV3_(utils)::getSIMDFlag();
+    }
+  else
+    simdFlag1 = flag1;
+  simdFlag2 = flag2;
   
+  simdSize = 1, flag1 = FV3_X86SIMD_FLAG_FPU, flag2 = FV3_X86SIMD_NULL;
+    
 #ifdef LIBFV3_FLOAT
-#if defined(ENABLE_3DNOW)
-  if((simdFlag&FV3_FLAG_3DNOW))
-    MULT_M = MULT_M_F_3DNOW;
-#endif
-#if defined(ENABLE_SSE)
-  if((simdFlag&FV3_FLAG_SSE))
-    MULT_M = MULT_M_F_SSE;
-#endif
-#if defined(ENABLE_SSE_V2)||defined(ENABLE_SSE2)
-  if((simdFlag&FV3_FLAG_SSE))
-    MULT_M = MULT_M_F_SSE_V2;
-#endif
-#if defined(ENABLE_SSE3)||defined(ENABLE_SSE4)
-  if((simdFlag&FV3_FLAG_SSE3))
-    MULT_M = MULT_M_F_SSE3;
-#endif
-#if defined(ENABLE_AVX)
-  if((simdFlag&FV3_FLAG_AVX))
-    MULT_M = MULT_M_F_AVX;
-#endif
-#if defined(ENABLE_FMA3)
-  if((simdFlag&FV3_FLAG_FMA3))
-    MULT_M = MULT_M_F_FMA3;
-#endif
-#if defined(ENABLE_FMA4)
-  if((simdFlag&FV3_FLAG_FMA4))
-    MULT_M = MULT_M_F_FMA4;
+#ifdef ENABLE_X86SIMD
+  if(simdFlag1&FV3_X86SIMD_FLAG_3DNOWP)simdSize = 2, flag1 = FV3_X86SIMD_FLAG_3DNOWP;
+  if(simdFlag1&FV3_X86SIMD_FLAG_SSE)   simdSize = 1, flag1 = FV3_X86SIMD_FLAG_SSE;
+  if(simdFlag1&FV3_X86SIMD_FLAG_SSE3)  simdSize = 1, flag1 = FV3_X86SIMD_FLAG_SSE3;
+  if(simdFlag1&FV3_X86SIMD_FLAG_AVX)   simdSize = 8, flag1 = FV3_X86SIMD_FLAG_AVX;
+  if(simdFlag1&FV3_X86SIMD_FLAG_FMA3)  simdSize = 8, flag1 = FV3_X86SIMD_FLAG_FMA3;
+  if(simdFlag1&FV3_X86SIMD_FLAG_FMA4)  simdSize = 8, flag1 = FV3_X86SIMD_FLAG_FMA4;
+
+  // override SSE_V1 option
+  if((simdFlag1&FV3_X86SIMD_FLAG_SSE)&&(simdFlag2&FV3_X86SIMD_FLAG_SSE_V1))
+    simdSize = 4, flag1 = FV3_X86SIMD_FLAG_SSE, flag2 = FV3_X86SIMD_FLAG_SSE_V1;
 #endif
 #endif
 
 #ifdef LIBFV3_DOUBLE
-#if defined(ENABLE_SSE2)||defined(ENABLE_SSE3)
-  if((simdFlag&FV3_FLAG_SSE2))
-    MULT_M = MULT_M_D_SSE2;
-#endif
-#if defined(ENABLE_SSE4)
-  if((simdFlag&FV3_FLAG_SSE4_1))
-    MULT_M = MULT_M_D_SSE4;
-#endif
-#if defined(ENABLE_AVX)
-  if((simdFlag&FV3_FLAG_AVX))
-    MULT_M = MULT_M_D_AVX;
-#endif
-#if defined(ENABLE_FMA3)
-  if((simdFlag&FV3_FLAG_FMA3))
-    MULT_M = MULT_M_D_FMA3;
-#endif
-#if defined(ENABLE_FMA4)
-  if((simdFlag&FV3_FLAG_FMA4))
-    MULT_M = MULT_M_D_FMA4;
+#ifdef ENABLE_X86SIMD
+  if(simdFlag1&FV3_X86SIMD_FLAG_SSE2)  simdSize = 2, flag1 = FV3_X86SIMD_FLAG_SSE2;
+  if(simdFlag1&FV3_X86SIMD_FLAG_SSE4_1)simdSize = 1, flag1 = FV3_X86SIMD_FLAG_SSE4_1;
+  if(simdFlag1&FV3_X86SIMD_FLAG_AVX)   simdSize = 4, flag1 = FV3_X86SIMD_FLAG_AVX;
+  if(simdFlag1&FV3_X86SIMD_FLAG_FMA3)  simdSize = 4, flag1 = FV3_X86SIMD_FLAG_FMA3;
+  if(simdFlag1&FV3_X86SIMD_FLAG_FMA4)  simdSize = 4, flag1 = FV3_X86SIMD_FLAG_FMA4;
 #endif
 #endif
+  simdFlag1 = flag1, simdFlag2 = flag2;
 }
 
-void FV3_(frag)::MULT(const fv3_float_t * iL, const fv3_float_t * iR,
-		      fv3_float_t * oL, fv3_float_t * oR)
+void FV3_(frag)::setSIMD(uint32_t flag1, uint32_t flag2)
 {
-  if(fragmentSize == 0) return;    
-#pragma omp parallel
-#pragma omp sections
-  {
-#pragma omp section
-    MULT_M(iL, fftImpulse.L, oL, fragmentSize);
-#pragma omp section
-    MULT_M(iR, fftImpulse.R, oR, fragmentSize);
-  }
+  // flag1 == NULL or unsupported Instruction -> Autodetect
+  if(flag1 == FV3_X86SIMD_NULL)
+    simdFlag1 = FV3_(utils)::getSIMDFlag();
+  else if(!(flag1 & FV3_(utils)::getSIMDFlag()))
+    {
+      std::fprintf(stderr, "frag::setSIMD(%08x): not supported, autodetected.\n", flag1);
+      simdFlag1 = FV3_(utils)::getSIMDFlag();
+    }
+  else
+    simdFlag1 = flag1;
+  simdFlag2 = flag2;
+
+  MULT_M = MULT_M_FPU, flag1 = FV3_X86SIMD_FLAG_FPU, flag2 = FV3_X86SIMD_NULL;
+    
+#ifdef LIBFV3_FLOAT
+#ifdef ENABLE_X86SIMD
+  if(simdFlag1&FV3_X86SIMD_FLAG_3DNOWP)MULT_M = MULT_M_F_3DNOW,  flag1 = FV3_X86SIMD_FLAG_3DNOWP;
+  if(simdFlag1&FV3_X86SIMD_FLAG_SSE)   MULT_M = MULT_M_F_SSE_V2, flag1 = FV3_X86SIMD_FLAG_SSE;
+  if(simdFlag1&FV3_X86SIMD_FLAG_SSE3)  MULT_M = MULT_M_F_SSE3,   flag1 = FV3_X86SIMD_FLAG_SSE3;
+  if(simdFlag1&FV3_X86SIMD_FLAG_AVX)   MULT_M = MULT_M_F_AVX,    flag1 = FV3_X86SIMD_FLAG_AVX;
+  if(simdFlag1&FV3_X86SIMD_FLAG_FMA3)  MULT_M = MULT_M_F_FMA3,   flag1 = FV3_X86SIMD_FLAG_FMA3;
+  if(simdFlag1&FV3_X86SIMD_FLAG_FMA4)  MULT_M = MULT_M_F_FMA4,   flag1 = FV3_X86SIMD_FLAG_FMA4;
+
+  // override SSE_V1 option
+  if((simdFlag1&FV3_X86SIMD_FLAG_SSE)&&(simdFlag2&FV3_X86SIMD_FLAG_SSE_V1))
+    MULT_M = MULT_M_F_SSE, flag1 = FV3_X86SIMD_FLAG_SSE, flag2 = FV3_X86SIMD_FLAG_SSE_V1;
+#endif
+#endif
+
+#ifdef LIBFV3_DOUBLE
+#ifdef ENABLE_X86SIMD
+  if(simdFlag1&FV3_X86SIMD_FLAG_SSE2)  MULT_M = MULT_M_D_SSE2, flag1 = FV3_X86SIMD_FLAG_SSE2;
+  if(simdFlag1&FV3_X86SIMD_FLAG_SSE4_1)MULT_M = MULT_M_D_SSE4, flag1 = FV3_X86SIMD_FLAG_SSE4_1;
+  if(simdFlag1&FV3_X86SIMD_FLAG_AVX)   MULT_M = MULT_M_D_AVX,  flag1 = FV3_X86SIMD_FLAG_AVX;
+  if(simdFlag1&FV3_X86SIMD_FLAG_FMA3)  MULT_M = MULT_M_D_FMA3, flag1 = FV3_X86SIMD_FLAG_FMA3;
+  if(simdFlag1&FV3_X86SIMD_FLAG_FMA4)  MULT_M = MULT_M_D_FMA4, flag1 = FV3_X86SIMD_FLAG_FMA4;
+#endif
+#endif
+  simdFlag1 = flag1, simdFlag2 = flag2;
+}
+
+void FV3_(frag)::MULT(const fv3_float_t * iL, fv3_float_t * oL)
+{
+  if(fragmentSize == 0) return;
+  MULT_M(iL, fftImpulse.L, oL, fragmentSize);
   return;
 }
 
-void FV3_(frag)::getFFT(fv3_float_t * oL, fv3_float_t * oR)
+void FV3_(frag)::getFFT(fv3_float_t * oL)
 {
   if(fragmentSize == 0) return;    
   std::memcpy(oL, fftImpulse.L, sizeof(fv3_float_t)*fragmentSize*2);
-  std::memcpy(oR, fftImpulse.R, sizeof(fv3_float_t)*fragmentSize*2);
 }
 
 long FV3_(frag)::getFragmentSize()
