@@ -2,7 +2,7 @@
  *  Impulse Response Processor model implementation
  *  Low Latency Version / Windows Multithread Version
  *
- *  Copyright (C) 2014 Teru Kamogashira
+ *  Copyright (C) 2006-2018 Teru Kamogashira
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,27 +41,27 @@ static unsigned __stdcall lfThread(void *vdParam)
       ResetEvent(event_StartThread);
 
       if(*info->flags & FV3_THREAD_FLAG_RUN)
-	{
-	  EnterCriticalSection(info->threadSection);
-	  if(info->lFragments->size() > 0)
-	    {
-	      for(long i = 0;i < (long)info->lFragments->size()-1;i ++)
-		{
-		  if((long)info->lFragments->size() > i+1)
-		    {
-		      info->lFragments->at(i+1)->MULT(info->lBlockDelayL->get(i), info->lBlockDelayR->get(i), *info->lSwapL, *info->lSwapR);
-		    }
-		}
-	    }
-	  *info->flags ^= FV3_THREAD_FLAG_RUN;
-	  SetEvent(event_ThreadEnded);
-	  LeaveCriticalSection(info->threadSection);
-	}
+        {
+          EnterCriticalSection(info->threadSection);
+          if(info->lFragments->size() > 0)
+            {
+              for(long i = 0;i < (long)info->lFragments->size()-1;i ++)
+                {
+                  if((long)info->lFragments->size() > i+1)
+                    {
+                      info->lFragments->at(i+1)->MULT(info->lBlockDelayL->get(i), *info->lSwapL);
+                    }
+                }
+            }
+          *info->flags ^= FV3_THREAD_FLAG_RUN;
+          SetEvent(event_ThreadEnded);
+          LeaveCriticalSection(info->threadSection);
+        }
       if(*info->flags & FV3_THREAD_FLAG_EXIT)
-	{
-	  *info->flags ^= FV3_THREAD_FLAG_EXIT;
-	  break;
-	}
+        {
+          *info->flags ^= FV3_THREAD_FLAG_EXIT;
+          break;
+        }
     }
   CloseHandle(event_StartThread);
   CloseHandle(event_ThreadEnded);
@@ -69,20 +69,17 @@ static unsigned __stdcall lfThread(void *vdParam)
   return 0;
 }
 
-FV3_(irmodel3w)::FV3_(irmodel3w)()
+// irmodel3wm
+
+FV3_(irmodel3wm)::FV3_(irmodel3wm)()
 {
-#ifdef DEBUG
-  fprintf(stderr, "irmodel3w::irmodel3w()\n");
-#endif
   validThread = false;
   threadId = 0;
   threadPriority = THREAD_PRIORITY_NORMAL;
   hostThreadData.lFragmentSize = &lFragmentSize;
   hostThreadData.lFragments = &lFragments;
   hostThreadData.lBlockDelayL = &lBlockDelayL;
-  hostThreadData.lBlockDelayR = &lBlockDelayR;
   hostThreadData.lSwapL = &lSwapSlot.L;
-  hostThreadData.lSwapR = &lSwapSlot.R;
   hostThreadData.flags = &threadFlags;
   hostThreadData.threadSection = &threadSection;
   InitializeCriticalSection(&mainSection);
@@ -91,18 +88,15 @@ FV3_(irmodel3w)::FV3_(irmodel3w)()
   resume();
 }
 
-FV3_(irmodel3w)::FV3_(~irmodel3w)()
+FV3_(irmodel3wm)::FV3_(~irmodel3wm)()
 {
-#ifdef DEBUG
-  fprintf(stderr, "irmodel3w::~irmodel3w()\n");
-#endif
   // REAPER does not call suspend().
   suspend();
   DeleteCriticalSection(&threadSection);
   DeleteCriticalSection(&mainSection);
 }
 
-bool FV3_(irmodel3w)::setLFThreadPriority(int priority)
+bool FV3_(irmodel3wm)::setLFThreadPriority(int priority)
 {
   bool ret = false;
   EnterCriticalSection(&mainSection);
@@ -112,7 +106,7 @@ bool FV3_(irmodel3w)::setLFThreadPriority(int priority)
   return ret;
 }
 
-void FV3_(irmodel3w)::resume()
+void FV3_(irmodel3wm)::resume()
 {
   EnterCriticalSection(&mainSection);
   if(validThread == false)
@@ -134,7 +128,7 @@ void FV3_(irmodel3w)::resume()
   LeaveCriticalSection(&mainSection);
 }
 
-void FV3_(irmodel3w)::suspend()
+void FV3_(irmodel3wm)::suspend()
 {
   EnterCriticalSection(&mainSection);
   if(validThread == true)
@@ -152,15 +146,15 @@ void FV3_(irmodel3w)::suspend()
   LeaveCriticalSection(&mainSection);
 }
 
-void FV3_(irmodel3w)::loadImpulse(fv3_float_t * inputL, fv3_float_t * inputR, long size)
-		     throw(std::bad_alloc)
+void FV3_(irmodel3wm)::loadImpulse(const fv3_float_t * inputL, long size)
+  throw(std::bad_alloc)
 {
   suspend();
   EnterCriticalSection(&mainSection);
   EnterCriticalSection(&threadSection);
   try
     {
-      FV3_(irmodel3)::loadImpulse(inputL, inputR, size);
+      FV3_(irmodel3m)::loadImpulse(inputL, size);
     }
   catch(std::bad_alloc)
     {
@@ -173,38 +167,36 @@ void FV3_(irmodel3w)::loadImpulse(fv3_float_t * inputL, fv3_float_t * inputR, lo
   resume();
 }
 
-void FV3_(irmodel3w)::unloadImpulse()
+void FV3_(irmodel3wm)::unloadImpulse()
 {
   suspend();
   EnterCriticalSection(&mainSection);
   EnterCriticalSection(&threadSection);
-  FV3_(irmodel3)::unloadImpulse();
+  FV3_(irmodel3m)::unloadImpulse();
   LeaveCriticalSection(&threadSection);
   LeaveCriticalSection(&mainSection);
   resume();
 }
 
-void FV3_(irmodel3w)::setFragmentSize(long size, long factor)
+void FV3_(irmodel3wm)::setFragmentSize(long size, long factor)
 {
   EnterCriticalSection(&mainSection);
   EnterCriticalSection(&threadSection);
-  FV3_(irmodel3)::setFragmentSize(size, factor);
+  FV3_(irmodel3m)::setFragmentSize(size, factor);
   LeaveCriticalSection(&threadSection);
   LeaveCriticalSection(&mainSection);
 }
 
-void FV3_(irmodel3w)::mute()
+void FV3_(irmodel3wm)::mute()
 {
   EnterCriticalSection(&mainSection);
   EnterCriticalSection(&threadSection);
-  FV3_(irmodel3)::mute();
+  FV3_(irmodel3m)::mute();
   LeaveCriticalSection(&threadSection);
   LeaveCriticalSection(&mainSection);
 }
 
-void FV3_(irmodel3w)::processZL(fv3_float_t *inputL, fv3_float_t *inputR,
-				fv3_float_t *outputL, fv3_float_t *outputR,
-				long numsamples, unsigned options)
+void FV3_(irmodel3wm)::processZL(fv3_float_t *inputL, long numsamples)
 {
   // SOL2 calls suspend() many times though the thread mutex is not clear.
   EnterCriticalSection(&mainSection);
@@ -224,9 +216,8 @@ void FV3_(irmodel3w)::processZL(fv3_float_t *inputL, fv3_float_t *inputR,
 
       EnterCriticalSection(&threadSection);
       lBlockDelayL.push(lIFFTSlot.L);
-      lBlockDelayR.push(lIFFTSlot.R);
-      lFragments[0]->MULT(lBlockDelayL.get(0), lBlockDelayR.get(0), lSwapSlot.L, lSwapSlot.R);
-      lFragmentsFFT.HC2R(lSwapSlot.L, lSwapSlot.R, lReverseSlot.L, lReverseSlot.R);
+      lFragments[0]->MULT(lBlockDelayL.get(0), lSwapSlot.L);
+      lFragmentsFFT.HC2R(lSwapSlot.L, lReverseSlot.L);
       lSwapSlot.mute(lFragmentSize*2);
       LeaveCriticalSection(&threadSection);
       
@@ -238,58 +229,31 @@ void FV3_(irmodel3w)::processZL(fv3_float_t *inputL, fv3_float_t *inputR,
   if(Scursor == 0)
     {
       sFramePointerL = lFrameSlot.L+Lcursor;
-      sFramePointerR = lFrameSlot.R+Lcursor;
       sSwapSlot.mute(sFragmentSize*2);
       sBlockDelayL.push(sIFFTSlot.L);
-      sBlockDelayR.push(sIFFTSlot.R);
-      for(long i = 1;i < (long)sFragments.size();i ++)
-	{
-	  sFragments[i]->MULT(sBlockDelayL.get(i-1), sBlockDelayR.get(i-1), sSwapSlot.L, sSwapSlot.R);
-	}
+      for(long i = 1;i < (long)sFragments.size();i ++){ sFragments[i]->MULT(sBlockDelayL.get(i-1), sSwapSlot.L); }
     }
   
   sOnlySlot.mute(sFragmentSize);
   
-  if(options & FV3_IR_MONO2STEREO)
-    {
-      for(long i = 0;i < numsamples;i ++)
-	{
-	  (lFrameSlot.L+Lcursor)[i] = (lFrameSlot.R+Lcursor)[i]
-	    = (sOnlySlot.L+Scursor)[i] = (sOnlySlot.R+Scursor)[i]
-	    = (inputL[i] + inputR[i])/2.0;
-	}
-    }
-  else
-    {
-      memcpy(lFrameSlot.L+Lcursor, inputL, sizeof(fv3_float_t)*numsamples);
-      memcpy(lFrameSlot.R+Lcursor, inputR, sizeof(fv3_float_t)*numsamples);
-      memcpy(sOnlySlot.L+Scursor, inputL, sizeof(fv3_float_t)*numsamples);
-      memcpy(sOnlySlot.R+Scursor, inputR, sizeof(fv3_float_t)*numsamples);
-    }
+  memcpy(lFrameSlot.L+Lcursor, inputL, sizeof(fv3_float_t)*numsamples);
+  memcpy(sOnlySlot.L+Scursor, inputL, sizeof(fv3_float_t)*numsamples);
   
   if(sFragments.size() > 0)
     {
-      sFragmentsFFT.R2HC(sOnlySlot.L, sOnlySlot.R, sIFFTSlot.L, sIFFTSlot.R);
-      sFragments[0]->MULT(sIFFTSlot.L, sIFFTSlot.R, sSwapSlot.L, sSwapSlot.R);
+      sFragmentsFFT.R2HC(sOnlySlot.L, sIFFTSlot.L);
+      sFragments[0]->MULT(sIFFTSlot.L, sSwapSlot.L);
       sReverseSlot.mute(sFragmentSize*2);
-      sFragmentsFFT.HC2R(sSwapSlot.L, sSwapSlot.R, sReverseSlot.L, sReverseSlot.R);
+      sFragmentsFFT.HC2R(sSwapSlot.L, sReverseSlot.L);
     }
   
   if(lFragments.size() > 0)
     {
-      for(long i = 0;i < numsamples;i ++)
-	{
-	  outputL[i] = (sReverseSlot.L+Scursor)[i] + (restSlot.L+Scursor)[i] + (lReverseSlot.L+Lcursor)[i];
-	  outputR[i] = (sReverseSlot.R+Scursor)[i] + (restSlot.R+Scursor)[i] + (lReverseSlot.R+Lcursor)[i];
-	}
+      for(long i = 0;i < numsamples;i ++){ inputL[i] = (sReverseSlot.L+Scursor)[i] + (restSlot.L+Scursor)[i] + (lReverseSlot.L+Lcursor)[i]; }
     }
   else
     {
-      for(long i = 0;i < numsamples;i ++)
-	{
-	  outputL[i] = (sReverseSlot.L+Scursor)[i] + (restSlot.L+Scursor)[i];
-	  outputR[i] = (sReverseSlot.R+Scursor)[i] + (restSlot.R+Scursor)[i];
-	}
+      for(long i = 0;i < numsamples;i ++){ inputL[i] = (sReverseSlot.L+Scursor)[i] + (restSlot.L+Scursor)[i]; }
     }
   
   Scursor += numsamples;
@@ -299,25 +263,126 @@ void FV3_(irmodel3w)::processZL(fv3_float_t *inputL, fv3_float_t *inputR,
 
   if(Scursor == sFragmentSize&&sFragments.size() > 0)
     {
-      sFragmentsFFT.R2HC(sFramePointerL, sFramePointerR, sIFFTSlot.L, sIFFTSlot.R);
+      sFragmentsFFT.R2HC(sFramePointerL, sIFFTSlot.L);
       memcpy(restSlot.L, sReverseSlot.L+sFragmentSize, sizeof(fv3_float_t)*(sFragmentSize-1));
-      memcpy(restSlot.R, sReverseSlot.R+sFragmentSize, sizeof(fv3_float_t)*(sFragmentSize-1));
       Scursor = 0;
     }
   
   if(Lcursor == lFragmentSize)
     {
       if(lFragments.size() > 0)
-	{
-	  lFragmentsFFT.R2HC(lFrameSlot.L, lFrameSlot.R, lIFFTSlot.L, lIFFTSlot.R);
-	  memcpy(lReverseSlot.L, lReverseSlot.L+lFragmentSize, sizeof(fv3_float_t)*(lFragmentSize-1));
-	  memcpy(lReverseSlot.R, lReverseSlot.R+lFragmentSize, sizeof(fv3_float_t)*(lFragmentSize-1));
-	}
+        {
+          lFragmentsFFT.R2HC(lFrameSlot.L, lIFFTSlot.L);
+          memcpy(lReverseSlot.L, lReverseSlot.L+lFragmentSize, sizeof(fv3_float_t)*(lFragmentSize-1));
+        }
       // thread enter
       Lcursor = Lstep = 0;
     }
   LeaveCriticalSection(&mainSection);
 }
 
-#include "freeverb/fv3_ns_end.h"
+// irmodel3w
 
+FV3_(irmodel3w)::FV3_(irmodel3w)()
+{
+  delete irmL, irmL = NULL;
+  delete irmR, irmR = NULL;
+  try
+    {
+      ir3wmL = new FV3_(irmodel3wm);
+      ir3wmR = new FV3_(irmodel3wm);
+      irmL = ir3mL = ir3wmL;
+      irmR = ir3mR = ir3wmR;
+    }
+  catch(std::bad_alloc)
+    {
+      delete irmL;
+      delete irmR;
+      throw;
+    }
+  InitializeCriticalSection(&mainSection);
+  // REAPER only calls resume() on on/off load.
+  resume();
+}
+
+FV3_(irmodel3w)::FV3_(~irmodel3w)()
+{
+  // REAPER does not call suspend().
+  suspend();
+  DeleteCriticalSection(&mainSection);
+}
+
+void FV3_(irmodel3w)::loadImpulse(const fv3_float_t * inputL, const fv3_float_t * inputR, long size)
+  throw(std::bad_alloc)
+{
+  suspend();
+  EnterCriticalSection(&mainSection);
+  try
+    {
+      FV3_(irmodel3)::loadImpulse(inputL, inputR, size);
+    }
+  catch(std::bad_alloc)
+    {
+      LeaveCriticalSection(&mainSection);
+      throw;
+    }
+  LeaveCriticalSection(&mainSection);
+  resume();
+}
+
+void FV3_(irmodel3w)::unloadImpulse()
+{
+  suspend();
+  EnterCriticalSection(&mainSection);
+  FV3_(irmodel3)::unloadImpulse();
+  LeaveCriticalSection(&mainSection);
+  resume();
+}
+
+void FV3_(irmodel3w)::resume()
+{
+  EnterCriticalSection(&mainSection);
+  FV3_(irbase)::resume();
+  LeaveCriticalSection(&mainSection);
+}
+
+void FV3_(irmodel3w)::suspend()
+{
+  EnterCriticalSection(&mainSection);
+  FV3_(irbase)::suspend();
+  LeaveCriticalSection(&mainSection);
+}
+
+void FV3_(irmodel3w)::mute()
+{
+  EnterCriticalSection(&mainSection);
+  FV3_(irmodel3)::mute();
+  LeaveCriticalSection(&mainSection);
+}
+
+void FV3_(irmodel3w)::setFragmentSize(long size, long factor)
+{
+  EnterCriticalSection(&mainSection);
+  FV3_(irmodel3)::setFragmentSize(size, factor);
+  LeaveCriticalSection(&mainSection);
+}
+
+void FV3_(irmodel3w)::setInitialDelay(long numsamples)
+  throw(std::bad_alloc)
+{
+  EnterCriticalSection(&mainSection);
+  FV3_(irbase)::setInitialDelay(numsamples);
+  LeaveCriticalSection(&mainSection);
+}
+
+bool FV3_(irmodel3w)::setLFThreadPriority(int priority)
+{
+  bool ret = false;
+  EnterCriticalSection(&mainSection);
+  if(ir3wmL->setLFThreadPriority(priority)) ret = true;
+  if(ir3wmR->setLFThreadPriority(priority)) ret = true;
+  LeaveCriticalSection(&mainSection);
+  return ret;
+}
+
+#include "freeverb/fv3_ns_end.h"
